require('myutil')

local version = 3

local whitelistTable = {}
local guardMode = "block"
local maxArea = 1000
local godTable = {}
local deconstructGuard = true
local spamTime = 60 * 60 -- Limit related message rate

-- cooldowns on alerts to limit admin spam. Thanks to /u/Mylon
local cooldowns = {}
function checkCooldown(playerName, action)
  if (not cooldowns[playerName]) then
    cooldowns[playerName] = {}
  end
  if (not cooldowns[playerName][action]) then
    cooldowns[playerName][action] = game.tick + spamTime
    return true
  elseif (cooldowns[playerName][action] > game.tick) then
    return false
  else
    cooldowns[playerName][action] = game.tick + spamTime
    return true
  end
end

-- god mode
function toggleGod(playerName)
  if (playerName == nil) then
    playerName = game.player.name
  end
  local player = getPlayerByName(playerName)
  if (player == nil) then
    game.player.print("Player " .. playerName .. " does not exist.")
    return
  end
  if (godTable[playerName]) then
    godTable[playerName] = false
    disableGod(player)
  else
    godTable[playerName] = true
    enableGod(player)
  end
end
function enableGod(player)
  player.cheat_mode = true
  if (player.character ~= nil) then
    player.character.destroy()
    player.character = nil
    player.print("You are now in god mode!")
  end
end
function disableGod(player)
	player.cheat_mode = false
  player.create_character()
  player.print("You are no longer in god mode.")
end
addCommand('god', 'player> - Toggles god mode for a player.', toggleGod)

function enableGod(player)
  player.cheat_mode = true
  if (player.character ~= nil) then
    player.character.destroy()
    player.character = nil
    player.print("You are now in god mode!")
  end
end

-- remove all decontruct marking
function _removeAllDeconstruct(force, surface, area)
  if (area == nil) then
    for key, entity in pairs(surface.find_entities()) do
        if entity.to_be_deconstructed(force) then
            entity.cancel_deconstruction(force)
        end
    end
  else
    for key, entity in pairs(surface.find_entities(area)) do
        if entity.to_be_deconstructed(force) then
            entity.cancel_deconstruction(force)
        end
    end
  end
end

function removeAllDeconstruct()
  printToAll("Admin " .. game.player.name  .. " is removing a deconstruct grief, please wait...")
  _removeAllDeconstruct(game.player.force, game.player.surface)
  printToAll("Deconstruct grief removed by " .. game.player.name)
end
addCommand('ungrief', ' - Removes all deconstruct markers.', removeAllDeconstruct)

-- deconstruct guard
function deconstructGuardToggle()
  if (deconstructGuard) then
    deconstructGuard = false
    game.player.print("DeconstructGuard disabled.")
  else
    deconstructGuard = true
    game.player.print("DeconstructGuard enabled.")
  end
end
addCommand('gtoggle', 'Toggle DeconstructGuard.', deconstructGuardToggle)

function guardWarn(offendingPlayer, width, height)
  printToAdmins(offendingPlayer.name .. " deconstructed a large area! (" .. width .. " x " .. height .. ")")
	printToAdmins("An admin must type /gwhitelist " .. offendingPlayer.name .. " to mute these alerts.")
end
function guardBlock(offendingPlayer, area, width, height)
  offendingPlayer.print("You cannot deconstruct that.")
  printToAdmins(offendingPlayer.name .. " attempted to deconstruct a large area! (" .. width .. " x " .. height .. ")", offendingPlayer.name)
  printToAdmins("An admin must type /gwhitelist " .. offendingPlayer.name .. " to allow this.")
  _removeAllDeconstruct(offendingPlayer.force, offendingPlayer.surface, area)
end
function onDeconstruct(event)
  if (event.item ~= "deconstruction-planner") then
    return
  end

  local area = event.area
  local width = area.right_bottom.x - area.left_top.x
  local height = area.right_bottom.y - area.left_top.y
  local areaNum = width * height

  local offendingPlayer = game.players[event.player_index]

  if (whitelistTable[offendingPlayer.name]) then
    return
  end

  if (areaNum > maxArea) then
    if (guardMode == "warning") then
      guardWarn(offendingPlayer, width, height)
    end
    if (guardMode == "block") then
      guardBlock(offendingPlayer, area, width, height)
    end
  end

  -- check for pumps being marked
  local entities = offendingPlayer.surface.find_entities_filtered{area=area, force=offendingPlayer.force, name="offshore-pump"}
  for key, entity in pairs(entities) do
    if entity.to_be_deconstructed(offendingPlayer.force) then
      printToAdmins(offendingPlayer.name .. " marked an offshore pump for deconstruct!")
      return
    end
  end
end
script.on_event(defines.events.on_player_deconstructed_area, onDeconstruct)

-- set maximum allowed deconstruct area (0 for none)
function setArea (area) 
  if (area == nil) then
    game.player.print("Must provide an argument")
    return
  end
  numArea = tonumber(area)

  if (numArea == nil) then
    game.player.print("First argument must be a number.")
    return
  end

  printToAdmins(game.player.name .. " set the max deconstruct area to " .. area)
  maxArea = numArea
end
addCommand('garea', '<area> - Set the maxmimum area to trigger DeconstructGuard.', maxArea)

-- whitelist a player to deconstruct
function whitelist (name)
  if (name == nil) then
    name = game.player.name
  end
  printToAdmins(game.player.name .. " deconstruct whitelisted " .. name)
  local player = getPlayerByName(name)
  if (player ~= nil) then
    player.print("You may now deconstruct freely.")
  end
	whitelistTable[name] = true
end
addCommand('gwhitelist', '<player> - Whitelist a player to deconstruct large areas.', whitelist)

-- blacklist a player to deconstruct
function blacklist (name)
  if (name == nil) then
    name = game.player.name
  end
  printToAdmins(game.player.name .. " deconstruct blacklisted " .. name)
  local player = getPlayerByName(name)
  if (player ~= nil) then
    player.print("You may no longer deconstruct freely.")
  end
	whitelistTable[name] = false
end
addCommand('gblacklist', '<player> - Blacklist a player from deconstructing large areas.', blacklist)

-- print the whitelist
function printWhitelist ()
  local any = false
  for name, allowed in pairs(whitelistTable) do
    if allowed then
      any = true
      game.player.print(name)
    end
  end
  if not any then
    game.player.print("No players are deconstruct whitelisted.")
  end
end
addCommand('glist', '- Show players who are whitelisted to deconstruct.', printWhitelist)

-- set guard mode
function setGuardMode (mode)
  if (mode == nil) then
    game.player.print("Must provide a deconstruct guard mode. Use \"/gmode block\" or \"/gmode warning\"")
    return
  end
  if (mode ~= "warning" and mode ~= "block") then
    game.player.print("Invalid deconstruct guard mode. Use \"/gmode block\" or \"/gmode warning\"")
    return
  end
  guardMode = mode
  printToAdmins("Deconstruct guard mode set to ".. mode)
end
addCommand('gmode', '<mode> - Set mode of DeconstructGuard. warning or block.', setGuardMode)

-- search inventories
function searchInv(itemName)
  if (itemName == nil) then
    game.player.print("Must specify an item name.")
  end
  local found = false
  for _, player in pairs(game.players) do
      local sum = 0
      for _, inv in pairs(getPlayerInventories(player)) do
        sum = sum + getItemCount(inv, itemName)
      end
      if (sum > 0) then
          found = true
          game.player.print(player.name .. " has " .. sum .. " " .. itemName .. "!")
      end
  end
  if (not found) then
    game.player.print("No player has that item.")
  end
end
addCommand('search', '<itemName> - Search all player inventories for an item', searchInv)

-- give item
function giveItem(itemName, count, playerName)
  if (itemName == nil) then
    game.player.print("Must specify an item name.")
    return
  end
  local numCount = tonumber(count)
  if (numCount == nil) then
    numCount = 1
  end
  if (playerName == nil) then
    playerName = game.player.name
  end
  local player = getPlayerByName(playerName)
  if (player == nil) then
    game.player.print("Player " .. playerName .. " does not exist.")
    return
  end

  local ok = insertItems(player, itemName, numCount)
  if (ok) then
    printToAdmins(game.player.name .. "gave " .. player.name .. " " .. numCount .. " " .. itemName)
  else
    game.player.print("Invalid item name.")
  end
end
addCommand("give", "<itemName> <count> <player> - Give a player items.", giveItem)

-- warning on pump mining: Thanks to /u/Mylon
function isWellPump(entity)
  return entity.name == "offshore-pump"
end
function isNuclearReactor(entity)
  return entity.name == "nuclear-reactor"
end
function isRocketSilo(entity)
  return entity.name == "rocket-silo"
end
-- removing blueprint: Thanks to /u/Mylon
function isBlueprint (entity)
  return entity.type == "entity-ghost" 
end
function onMine (event)
  if not event.entity and not event.entity.valid then
    return
  end

  local player = game.players[event.player_index]
  local entity = event.entity
  if (whitelistTable[player.name]) then
    return
  end

  if (isWellPump(entity)) then
    if (checkCooldown(player.name, "pump")) then
      printToAdmins(player.name .. " has mined a well pump.")
    end
  elseif (isNuclearReactor(entity)) then
    if (checkCooldown(player.name, "reactor")) then
      printToAdmins(player.name .. " has mined a nuclear reactor.")
    end
  elseif (isRocketSilo(entity)) then
    if (checkCooldown(player.name, "silo")) then
      printToAdmins(player.name .. " has mined a rocket silo")
    end
  elseif (isBlueprint(entity)) then
    if (checkCooldown(player.name, "blueprint")) then
      printToAdmins(player.name .. " removed a blueprint ghost.")
    end
  end
end
script.on_event(defines.events.on_player_mined_entity, onMine)

-- equip an atomic bomb: Thanks to /u/Mylon
function onAtomic(event)
  local player = game.players[event.player_index]

  if (whitelistTable[player.name]) then
    return
  end

  if player.get_item_count("atomic-bomb") > 0 then
    if checkCooldown(player.name, "atomic") then
      printToAdmins(player.name .. " has equipped an Atomic Bomb.")
    end
  end
end
script.on_event(defines.events.on_player_ammo_inventory_changed, onAtomic)


-- equip artillery remote: Thanks to /u/Mylon
function onArtRemote(event)
  local player = game.players[event.player_index]

  if (whitelistTable[player.name]) then
    return
  end

  if player.cursor_stack.valid_for_read and player.cursor_stack.name == "artillery-targeting-remote" then
    if checkCooldown(player.name, "artillery-remote") then
      printToAdmins(player.name .. " has equipped an Artillery Remote.")
    end
  end
end
script.on_event(defines.events.on_player_ammo_inventory_changed, onArtRemote)

-- general destruction: Thanks to /u/Mylon
function onDie(event)
  if not (event.entity and event.entity.valid) then
    return
  end
  if not (event.cause and event.cause.type == "player") then
    return
  end
  local player = event.cause.player

  if (not player.valid) then -- player kills themselves
    return
  end

  local entity = event.entity

  if (whitelistTable[player.name]) then
    return
  end

  if (player.force == entity.force) then 
    if (isWellPump(event.entity)) then
      if (checkCooldown(player.name, "pump")) then
        printToAdmins(player.name .. " has destroyed a well pump.")
      end
    elseif (isNuclearReactor(event.entity)) then
      if (checkCooldown(player.name, "reactor")) then
        printToAdmins(player.name .. " has destroyed a nuclear reactor.")
      end
    elseif (isRocketSilo(event.entity)) then
      if (checkCooldown(player.name, "silo")) then
        printToAdmins(player.name .. " has destroyed a rocket silo.")
      end
    elseif checkCooldown(player.name, "destroy") then
      printToAdmins(player.name .. " is destroying friendly entities.")
    end
  end
end
script.on_event(defines.events.on_entity_died, onDie)

-- set spam timer
function setSpamTimer(time)
  if (time == nil) then
    game.player.print("Must provide an argument.")
    return
  end
  numTime = tonumber(time)
  if (numTime == nil) then
    game.player.print("First argument must be a number.")
    return
  end
  spameTime = numTime * 60
  printToAdmins("Set the spam limit to 1 related message per " .. time .. " seconds.")
end
addCommand("setspamtime", "<time> - Time (in seconds) to limit related messages (default 60s).", setSpamTimer)