function addCommand(name, help, func)
    commands.add_command(
        name,
        help,
        function(event)
            local player = game.players[event.player_index]
            if not player.admin then
                player.print("Only admins may use commands.")
                return
            end
            if event.parameter == nil then
                event.parameter = ""
            end
            local params = splitString(event.parameter)
            func(params[1], params[2], params[3], params[4], params[5], params[6])
        end
    )
end

function splitString(s)
    chunks = {}
    count = 0
    for substring in s:gmatch("%S+") do
        count = count + 1
        chunks[count] = substring
    end
    return chunks
end

function printToAll(message)
    for _, player in pairs(game.players) do
        player.print(message)
    end
end

function printToAdmins(message)
    for _, player in pairs(game.players) do
        if (player.admin) then
            player.print(message)
        end
    end
end

function printToAllBut(message, excludeName)
    for _, player in pairs(game.players) do
        if (player.name ~= excludeName) then
            player.print(message)
        end
    end
end

function getPlayerByName(playerName)
    for _, player in pairs(game.players) do
        if (player.name == playerName) then
            return player
        end
    end
end

function getPlayerInventories(player)
    local invs = {}

    invs['quickbar'] = player.get_inventory(defines.inventory.player_quickbar)
    invs['main'] = player.get_inventory(defines.inventory.player_main)
    invs['guns'] = player.get_inventory(defines.inventory.player_guns)
    invs['ammo'] = player.get_inventory(defines.inventory.player_ammo)
    invs['tools'] = player.get_inventory(defines.inventory.player_tools)

    if player.get_inventory(defines.inventory.player_armor) then
        invs['armor'] = player.get_inventory(defines.inventory.player_armor)
    end

    if player.get_inventory(defines.inventory.player_vehicle) then
        invs['vehicle'] = player.get_inventory(defines.inventory.player_vehicle)
    end

    if player.get_inventory(defines.inventory.player_trash) then
        invs['trash'] = player.get_inventory(defines.inventory.player_trash)
    end

    return invs
end


function getItemCount(inventory, itemName)
    local ok, result = pcall(inventory.get_item_count, itemName)
    if ok then
        return result
    else
        return 0
    end
end

function insertItems(inventory, itemName, count)
    local ok, result = pcall(inventory.insert, {name = itemName, count = count})
    if ok then
        return true
    else
        return false
    end
end